http = require 'http'
fs = require 'fs' 
path = require 'path'
spawn = require('child_process').spawn
mime = require 'mime'

tumbler = require "tumbler"

DEFAULT_CONFIG =
    port: 10000
    path: process.cwd()



ARGUMENTS = [
    [ ["-p", "--port"], "port", true, "Server port" ],
    [ ["-d", "--directory"], "path", true, "Directory" ],
    [ ["--deploy"], "deploy", true, "Deploy" ],
    [ ["-h", "--help" ], "help", false, "Print this help" ],
]


serve_static_file = (response,filepath)->
    try
        if path.existsSync filepath
            stat =  fs.statSync filepath
            if stat.isDirectory()
                response.writeHead 404
                response.end "That's a directory."  
            else
                content_type =  mime.lookup filepath
                response.writeHead 200,
                    'content-type': content_type
                file = fs.createReadStream filepath
                file.pipe response
        else
            response.writeHead 404
            response.end "File not found"
    catch error
        response.writeHead 500
        response.end "Error "+error_msg     

request_handler = (builder,config)->
    (request, response)->
        response.setHeader "Access-Control-Allow-Origin", "*"
        url = request.url
        console.log 'GET ', url
        url_base = url.split '?', 1
        filepath = config.path + url_base
        serve_file = ->
            serve_static_file response,filepath
        failure_callback = (err_msg)->
            response.writeHead 500
            response.end err_msg 
        builder.build filepath,serve_file,failure_callback

merge_dic = (d,d2)->
    for k,v of d2
        d[k] = v
    d


compare_path = (left,right) ->
    fs.realpathSync(left) != fs.realpathSync(right) 
    

argument_parser = (argument_options_, default_config)->
    argument_options = argument_options_.slice 0
    argument_config = default_config
    parse_option_map = {}

    for [options, config_key, need_arg, help] in argument_options
        for option in options
            parse_option_map[option] = [config_key, need_arg]
    
    help: ->
        console.log "\nusage:"
        console.log ' ' + __filename + " [options]\n"
        console.log 'options :'
        for [options, config_key, need_arg, help] in argument_options
            if need_arg
                console.log '   ' + options + ' option : ' + help
            else
                console.log '   ' + options + ' : ' + help
        console.log ''
    
    parse: (argv)->
        config = merge_dic {},DEFAULT_CONFIG
        while (argv.length > 0)
            option = argv.shift()
            parse_options = parse_option_map[option]
            if parse_options?
                [config_key, need_arg] = parse_options
                if need_arg
                    config[config_key] = argv.shift()
                else
                    config[config_key] = true
            else
                throw "Option unknown : " + option
        config

ARGUMENTS_PARSER = argument_parser ARGUMENTS


prompt=(question,callback)->
    console.log question
    process.stdin.resume()
    process.stdin.on "data",(data)->
        process.stdin.pause()
        callback data.toString().trim()



visit_directory = (dir_path,action,cond)->
    # visits children before parents
    if cond? and cond dir_path
        console.log "ok", dir_path
        children = fs.readdirSync dir_path
        for child in children
            if not (/\..*/.test child)
                child_fullpath = path.join dir_path,child 
                child_stat = fs.statSync child_fullpath
                if child_stat.isDirectory()
                    visit_directory child_fullpath,action
                try
                    action child_fullpath
                catch msg
                    throw "Failed on " + child_fullpath
    else
        console.log "fail", dir_path

erase = (filepath)->
    file_stat = fs.statSync filepath
    if file_stat.isDirectory()
        fs.rmdirSync filepath
    else
        fs.unlinkSync filepath

yesno_prompt=(question,callback)->
    handle_answer=(answer)->
        is_yes = ( answer.toLowerCase() in ["y", "yes"] )
        callback is_yes
    prompt question + " [y/n]",handle_answer


prepare_directory = (directory_path, callback)->
    if path.existsSync directory_path
        content = fs.readdirSync directory_path
        if content.length == 0
            callback directory_path
        else
            console.log "Directory not empty"
            console.log directory_path
            yesno_prompt "Do you want to erase all of its content",(answer)->
                if (answer)
                    console.log "cleaning up"
                    visit_directory directory_path,erase
                else
                    throw "Aborting..."
    else
        try
            fs.mkdirSync directory_path,0777
        catch e
            throw "Could not make directory : " + directory_path
        callback directory_path

run_deploy = (config)->
    console.log "deploy not implemented yet."
    #dest_filepath = path.resolve config.deploy
    #prepare_directory dest_filepath,-> deploy config.path,dest_filepath

run_server = (builder,config)->
    server = http.createServer (request_handler builder,config)
    console.log 'Listening on port : ' + config.port
    server.listen(config.port)

main = ->
    builder = new tumbler.Builder()
    builder.register_compiler new tumbler.BundleCompiler()
    builder.register_compiler new tumbler.SimpleCompiler 'coffee', 'js', 'coffee', ['-p']
    try
        sprite = require 'tumbler-sprite'
        builder.register_compiler new sprite.SpriteCompiler()
    catch exception
        console.log "Couldn't load sprite"
    builder.register_compiler new tumbler.SimpleCompiler 'less', 'css', 'lessc',[]
    builder.register_compiler new tumbler.TrivialCompiler()
        
    try
        config = ARGUMENTS_PARSER.parse process.argv[2..process.argv.length]
        console.log config
        if config.help
            ARGUMENTS_PARSER.help()
        else if config.deploy
            run_deploy config
        else
            console.log "Run server"
            run_server builder,config
        
    catch e
        console.log e
        ARGUMENTS_PARSER.help()

if require.main == module
    main()


